#include<iostream.h>
#include<stdlib.h>
#include"mempl_out.h"
#include"btree.h"

bool TestStrategy::splitt_leavenode(LeafPageAccess &lpa)
{
  return lpa.count() >= 6;
}

bool TestStrategy::splitt_indexnode(IndexPageAccess &ipa)
{
  return ipa.count() >= 6;
}

void Btree::init(const char *filename)
{
  RAMS::RPointer<Managementinfo> mip;
  int neu=bpool.init(filename,mip);
  root.init(bpool.getseg(),mip.ptr()->btreeroot);
  first_page.init(bpool.getseg(),mip.ptr()->first_page);
  if(neu)
  {
    bpool.getpage(root);
    LeafPageAccess lpac(root.ptr());
    lpac.init();
    first_page = root;
  }
  else
  {
  }
}

int Btree::findpage(RAMS::RPointer<CommonPage,PAGESIZE>& ret,const octet_t *key,size_t keylen)
{
  IndexPageAccess ipa;
  int f;
  ret = root;
  while(ret.ptr()->pagetyp == indexpage)
  {
    ipa.setto(ret.ptr());
    f=ipa.find(key,keylen);
    if(f<ipa.count())
    {
      ret.setoff(ipa.off(f));
      cerr << "Page " << ret.ptr() << " key[" << f << "]==" << key << endl;
    }
    else
    {
      cerr << "Key " << key << " is the greatest ... " << endl;
      return -1;
    }
  }
  return 0;
}

mempl_t Btree::find(mempl_t key)
{
  RAMS::RPointer<CommonPage,PAGESIZE> akt;
  mempl_t ret;
  int f;
  int r = findpage(akt,key.ptr,key.l);
  if(r>=0)
  {
    LeafPageAccess lpa(akt.ptr());
    f=lpa.find(key.ptr,key.l);
    if(lpa.keycmp(f,key.ptr,key.l)==0)
    {
      return lpa.content(f);
    }
  }
  ret.l=0;
  ret.ptr=NULL;
  return ret;
}

void Btree::replacelargestkey(RAMS::RPointer<CommonPage,PAGESIZE>& ret,const octet_t *key,size_t keylen)
{
  IndexPageAccess ipa;
  int f;
  ret = root;
  while(ret.ptr()->pagetyp == indexpage)
  {
    ipa.setto(ret.ptr());
    ipa.replacelargestkey(key,keylen);
    ret.setoff(ipa.off(ipa.count()-1));
  }
}

void Btree::blattverkettung(off_t prev,off_t next,RAMS::RPointer<CommonPage,PAGESIZE>& pp1,RAMS::RPointer<CommonPage,PAGESIZE>& pp2)
{
    pp1.ptr()->next=pp2.offset();pp2.ptr()->prev=pp1.offset();
    pp1.ptr()->prev=prev;
    if(pp1.ptr()->prev == 0)
    {
      first_page=pp1;
    }
    else
    {
      RAMS::RPointer<CommonPage,PAGESIZE> vorg(prev,bpool.getseg());
      vorg.ptr()->next = pp1.offset();
    }
    pp2.ptr()->next=next;
    if(pp2.ptr()->next == 0)
    { // Lastpage   
    }
    else
    {
      RAMS::RPointer<CommonPage,PAGESIZE> nachf(next,bpool.getseg());
      nachf.ptr()->prev = pp2.offset();
    }
}

int Btree::insert(const octet_t *key,size_t keylen,const octet_t *cont,size_t len)
{
  RAMS::RPointer<CommonPage,PAGESIZE> akt;
  int r=findpage(akt,key,keylen);
  if(r<0) replacelargestkey(akt,key,keylen);
  cerr << "LeafPage " << akt.ptr() << endl;
  LeafPageAccess lpac(akt.ptr());
  /* Annahme: aufgrund unserer guten strategie ist immer Platz 
     sonst mu der Kunde es nochmals Versuchen */
  r=lpac.insert(key,keylen,cont,len);
 
  if(strategy->splitt_leavenode(lpac))
  {
    cerr << "Seite mu gesplittet werden ... " << endl;
    RAMS::RPointer<CommonPage,PAGESIZE> pp1(0,bpool.getseg());
    RAMS::RPointer<CommonPage,PAGESIZE> pp2(0,bpool.getseg());
    bpool.getpage(pp1);
    bpool.getpage(pp2);
    lpac.splitt(pp1.ptr(),pp2.ptr());
    LeafPageAccess lpac1(pp1.ptr());
    LeafPageAccess lpac2(pp2.ptr());
    
    blattverkettung(akt.ptr()->prev,akt.ptr()->next,pp1,pp2);
 
    /* Einfuegen der Schluessel in Indexknoten */
    RAMS::RPointer<CommonPage,PAGESIZE> parent(0,bpool.getseg());
    IndexPageAccess ipa;
    lpac.get_parent(parent);
    if(parent.offset()==0)
    {
      parent=akt;
      ipa.init(parent.ptr(),10);
      ipa.insert(lpac1.keymax(),pp1.offset());
      ipa.insert(lpac2.keymax(),pp2.offset());
      root=parent;
      lpac1.set_parent(parent);
      lpac2.set_parent(parent);
    }
    else
    {
      ipa.setto(parent.ptr());
      ipa.insert(lpac1.keymax(),pp1.offset());
      ipa.changeoffset(lpac2.keymax(),pp2.offset());
      bpool.free(akt);
    }
    return 0;
  }
  return r;
}



/*------------ Testprogramm --------------------*/
void usage()
{
  cerr << "Kommandos:" << endl;
  cerr << "\tinsert\t:Liest Stze von stdin. Tabulator trennt key und Satz\n";
  cerr << "\tlistall\t: Auflistung alle Blattknoten " << endl;
  cerr << "\tsearch <string> \t: suche von string " << endl;
  cerr << "\tdump\t: Ausdrucken der Speicherseiten" << endl;
}


void kommando_insert(Btree &tree)
{
  char puffer[2048];
  while(fgets(puffer,2048,stdin)!=0)
  {
    char *cont = strchr(puffer,'\t');
    if(cont == NULL) 
    {
      cerr << "Kein Inhalt ..." << endl;
      continue;
    }
    cont++;
    mempl_t key(cont-puffer-1,(octet_t *)puffer);
    mempl_t content(strlen(cont)-1,(octet_t *)cont);
    cerr << "Key :>" << key << '<' << endl;
    cerr << "Content:>" << content << '<' << endl;
    int r=tree.insert(key.ptr,key.l,content.ptr,content.l);
  }
}

void kommando_listall(Btree &tree)
{
  extern void LeafPageAccess_dump(LeafPageAccess &lpac,ostream& str);
  for(LeafPageIterator lpi(&tree);!lpi.done();lpi.advance())
  {
    cout << "Page " << lpi.pageptr() << endl;
    LeafPageAccess_dump(lpi.page(),cout);
  }
}

void kommando_search(Btree &tree,char *key)
{
  mempl_t mkey(strlen(key),(octet_t *)key);
  mempl_t ret = tree.find(mkey);
  if(ret.ptr!=NULL)
  {
    cout << ret << endl;
  }
  else
  {
    cerr << mkey << " nicht gefunden ! " << endl;
    exit(1);
  }
}

void kommando_dump(Btree& tree)
{
  extern void LeafPageAccess_dump(LeafPageAccess &lpac,ostream& str);
  extern void IndexPageAccess_dump(IndexPageAccess &lpac,ostream& str,char *base=0);
  RAMS::RPointer<CommonPage,PAGESIZE> akt(0,tree.bpool.getseg());
  cout << akt.ptr() << " Verwaltungsinformationen " << endl;
  for(int i=0;i<10;i++)
  {
    akt+=1;
    cout << akt.ptr();
    switch(akt.ptr()->pagetyp)
    {
    case freepage: cout << " Freepage" << endl; break;
    case leafPage:
      {
	cout << " LeafPage" << endl;
	LeafPageAccess lpac(akt.ptr());
	LeafPageAccess_dump(lpac,cout);
      }
      break;
    case indexpage:
      {
	cout << " Indexpage" << endl;
	IndexPageAccess ipac(akt.ptr());
	IndexPageAccess_dump(ipac,cout,(char *)(tree.bpool.getseg()->begin()));
      }
      break;
    default: 
      cout << " unbekannt " << akt.ptr()->pagetyp << endl;
    }
  }
}

Btree tree;
int main(int argc,char **argv)
{
  tree.init("baum.mf");
  
  if(argv[1]==NULL) { usage(); exit(1); }
  if(strcmp(argv[1],"insert")==0) kommando_insert(tree);
  else if(strcmp(argv[1],"listall")==0) kommando_listall(tree);
  else if(strcmp(argv[1],"search")==0 && argv[2]!=NULL)
                                          kommando_search(tree,argv[2]);
  else if(strcmp(argv[1],"dump")==0) kommando_dump(tree);
  else usage();
}




