/**
 * @file btree.h
 * @brief defines the relative Pointers 
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A> 2002
 *
 *
 */
#include"mempl.h"
#include"blockpool.h"
#include"leafpage.h"
#include"indexpage.h"

class Btree;

/**
 * @brief Iterator over all records
 * unimplemented now
 */
class Iterator
{
  Btree *btree_;
  RAMS::RPointer<CommonPage,PAGESIZE> akt_page;
  int index;
public:
  Iterator(Btree* tree);
  bool done();
  void advance();
  mempl_t key();
  mempl_t content();
};

/**
 * @brief Strategy Class
 */
class BtreeStrategy
{
public:
  virtual bool splitt_leavenode(LeafPageAccess &lpa)=0;
  virtual bool splitt_indexnode(IndexPageAccess &ipa)=0;
};

/**
 * @brief Strategy Class for Testing
 */
class TestStrategy : public BtreeStrategy
{
public:
  bool splitt_leavenode(LeafPageAccess &lpa);
  bool splitt_indexnode(IndexPageAccess &ipa);
} theTestStrategy;

/**
 * @brief Interface for our Btree
 */
class Btree
{
  struct Managementinfo
  {
    off_t btreeroot;
    off_t first_page;
  };
protected:
  Blockpool bpool;
  RAMS::RPointer<CommonPage,PAGESIZE> root;
  RAMS::RPointer<CommonPage,PAGESIZE> first_page;
  BtreeStrategy* strategy;
 
  /**
   * @returns 0 if ok, -1 if key to big 
   */
  int findpage(RAMS::RPointer<CommonPage,PAGESIZE>& ret,const octet_t *key,size_t keylen);
  /**
   * Wenn ein Knoten eingefuegt wird, der groer als alle bisherigen ist,
   * wird dieser in der letzen Seite eingetragen.
   * Der Maximalwert muss auch in alle Blaetter eingetragen werden.
   */
  void replacelargestkey(RAMS::RPointer<CommonPage,PAGESIZE>& ret,const octet_t *key,size_t keylen);

  /**
   * Vollzieht die Verkettung der Bltter
   * @param prev offset des Vorg&auml;gerknotens
   * @param next offset des Nachfolgerknotens
   * @param pp1  erster durch die Splittung entstandenen Knoten
   * @param pp2  zweiter durch die Splittung entstandenen Knoten
   */
  void blattverkettung(off_t prev,off_t next,RAMS::RPointer<CommonPage,PAGESIZE>& pp1,RAMS::RPointer<CommonPage,PAGESIZE>& pp2);

public:
  Btree():strategy(&theTestStrategy) {}
  void init(const char *filename);
  int insert(const octet_t *key,size_t keylen,const octet_t *cont,size_t l);
  mempl_t find(mempl_t key); // res.ptr == 0 if not found
  friend class LeafPageIterator;
  friend void kommando_dump(Btree &tree);
};

/**
 * @brief Iterator over all levapages
 */
class LeafPageIterator
{
  RAMS::RPointer<CommonPage,PAGESIZE> akt_page;
  LeafPageAccess lpac;
public:
  LeafPageIterator(Btree* tree)
  {
    akt_page=tree->first_page;
    lpac.setto(akt_page.ptr());
  }
  bool done() { return akt_page.offset()==0; }
  void advance()
  {
    lpac.next(akt_page);
    if(akt_page.offset()!=0)
    { lpac.setto(akt_page.ptr());
    }
  }
  LeafPageAccess& page() { return lpac; } 
  void *pageptr() { return akt_page.ptr(); }
};  
