/**
 * @file indexpage.h
 * @brief defines a Index page and the Access-Structure
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A> 2002
 *
 *
 */
#ifndef INDEX_PAGE_H
#define INDEX_PAGE_H
#include"allg.h"
#include<rpointer.h>
#include"mempl.h"
#include"keypage.h"

/**
 * \brief Indexpage for the B*-Tree
 * persistent layout
 */
struct Keyentry
{
  off_t offset;
  octet_t key[1];  /// keylength long, dynamic;
};

struct IndexPage : KeyPage
{
  uint16_t keylen;  // All keys have a Fixed length
  Keyentry keys[1]; // growing Array
};

class IndexPageAccess : public KeyPageAccess
{
public:
  IndexPageAccess():KeyPageAccess() {}
  IndexPageAccess(CommonPage* base):KeyPageAccess(base) {}
 
  virtual Page_type_t pagetype();
  virtual size_t space();
  /**
   * initialisiert leere Seite
   */
  void init(uint16_t keylen);
  void init(CommonPage *base,uint16_t keylen)
  {
    RAMS::Mseg::setto((char *)base,PAGESIZE);
    init(keylen);
  }

  int  maxcount() 
  {
    return (PAGESIZE-offsetof(IndexPage,keys))
               /(sizeof_Keyentry());
  }

  /**
   * F&uuml;gt Schl&uuml:ssel ein
   * @return -1 falls voll, index sonst
   */
  int insert(const octet_t *key,size_t keylen,off_t offset);
  int insert(mempl_t key,off_t offset)  { return insert(key.ptr,key.l,offset); }
  virtual mempl_t key(int i);
  virtual int keycmp(int i,const octet_t *other,size_t olen);

  off_t off(int i) { return keyentry(i)->offset; }
  void changeoffset(mempl_t key,off_t off)
  {
    int f = find(key);
    keyentry(f)->offset = off;
  }
  int replacelargestkey(const octet_t *key,size_t keylen);
  int splitt(CommonPage *neu1,CommonPage *neu2);
  
 
protected:
  IndexPage *page() { return (IndexPage *)begin(); }
  size_t sizeof_Keyentry() { return sizeof(off_t)+page()->keylen; }
  Keyentry* keyentry(int i)
  {
    return (Keyentry *)((octet_t *)(page()->keys)+i*sizeof_Keyentry());
  }
};
#endif



