/**
 * @file keypage.h
 * @brief defines a keypage and the Access-Structure
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A> 2002
 *
 */
#ifndef KEYPAGE_H
#define KEYPAGE_H
#include"allg.h"
#include<rpointer.h>
typedef off_t ploff_t;  // Local Offset in Page
#include"mempl.h"
#include"common_page.h"

struct KeyPage : CommonPage
{
  off_t parent;
  long anz;     /// number of entries in the Page
};

/**
 * The KeyPageAccess is a abstraction over the LeafPageAcess and
 * the IndexPageAccess.
 */
class KeyPageAccess: public RAMS::Mseg
{
public:
  KeyPageAccess():Mseg() {}
  KeyPageAccess(CommonPage * base)
    :Mseg((char *)base,PAGESIZE) {}
  void setto(CommonPage *base)
    {
      RAMS::Mseg::setto((char *)base,PAGESIZE);
      assert(page()->pagetyp == pagetype());
    }

  virtual Page_type_t pagetype()=0;
  int count() { return page()->anz; }
  virtual size_t space() = 0;
  virtual mempl_t key(int i)=0;
  mempl_t keymax() { return key(count()-1); }
  virtual int keycmp(int i,const octet_t *other,size_t olen)=0;

  int find(const octet_t *key,size_t keylen);
  int find(mempl_t key) { return find(key.ptr,key.l); }
  /**
   * Verkettung vorwaerts
   */
  void next(RAMS::RPointer<CommonPage,PAGESIZE>& akt)
  {
    akt.setoff(page()->next);
  }
  /**
   * Verkettung rueckwaerts
   */
  void prev(RAMS::RPointer<CommonPage,PAGESIZE>& akt)
  {
    akt.setoff(page()->prev);
  }
  /** 
   * Parent knoten 
   */
  void get_parent(RAMS::RPointer<CommonPage,PAGESIZE>& akt)
  {
    akt.setoff(page()->parent);
  }
  void set_parent(RAMS::RPointer<CommonPage,PAGESIZE>& akt)
  {
    page()->parent = akt.offset();
  }
  
  /**
   * @brief Teilen eines Knotens
   * Die Seiten neu1 und neu2 muessen vom Aufrufer irgendwie beschafft werden
   * @param neu1 Freie Seite1
   * @param neu2 Freie Seite2
   */
  virtual int splitt(CommonPage *neu1,CommonPage *neu2)=0;
protected:
  KeyPage *page() { return (KeyPage *)begin(); }
};
#endif


