#include"allg.h"
#include"leafpage.h"
#include"helpers.h"



LeafPage *LeafPage::init(char *m)
{
  LeafPage *ret = (LeafPage *)m;
  ret->pagetyp = leafPage;
  ret->parent=0;
  ret->anz=0;
  ret->last=PAGESIZE;
  return ret;
}

Page_type_t  LeafPageAccess::pagetype() { return leafPage; }

size_t LeafPageAccess::space() 
{
  return page()->last - page()->endoftable();
}

int LeafPageAccess::keycmp(int i,const octet_t *other,size_t olen)
{
  octet_t *mykey = (octet_t *)begin()+page()->records[i].key.off;
  return memcmp(mykey,other,min(page()->records[i].key.len,olen));
}

mempl_t LeafPageAccess::key(int i)
{
  mempl_t ret;
  ret.l = page()->records[i].key.len;
  ret.ptr = (octet_t *)begin()+page()->records[i].key.off;
  return ret;
}

int LeafPageAccess::insertstr(memp_t& memp,const octet_t *str,size_t len)
{
  if(len > space()) return -1;
  RAMS::RPointer<octet_t> loc(this,page()->last);
  loc-=len;
  memp.chksum = memcpy_chksum(loc.ptr(),str,len);
  memp.len=len;
  memp.off= loc.offset();
  return 0;
}


int LeafPageAccess::insert(const octet_t *key,size_t keylen,const octet_t *cont,size_t l)
{
  int f = find(key,keylen);
  
  if(keylen+l+sizeof(Record_t) > space()) return -1;  // voll;

  if(f<count())
  {
    if(keycmp(f,key,keylen)==0) return 1; // doppelt
    char *start=(char *)(page()->records+f);
    char *ziel =(char *)(page()->records+f+1);
    size_t len=(count()-f)*sizeof(Record_t);
    memmove(ziel,start,len);
  }  
  page()->anz++; 
  insertstr((page()->records[f].key),key,keylen);
  insertstr((page()->records[f].content.memp),cont,l);
  page()->records[f].content.is_blob=0;
  return f;
}

int LeafPageAccess::splitt(CommonPage *neu1,CommonPage *neu2)
{

  LeafPageAccess erste(neu1);
  LeafPageAccess zweite(neu2);
  erste.init();
  zweite.init();

  int g = count()/2;
  int i;
  for(i=0;i<g;i++)
  {
    mempl_t k = key(i);
    mempl_t cont = content(i);
    erste.insert(k.ptr,k.l,cont.ptr,cont.l);
  }
  for(i=g;i<count();i++)
  {
    mempl_t k = key(i);
    mempl_t cont = content(i);
    zweite.insert(k.ptr,k.l,cont.ptr,cont.l);
  }
  return 0;
}

#include<iostream.h>
#include<stdio.h>
#include<unistd.h>
#include"mempl_out.h"


void LeafPageAccess_dump(LeafPageAccess &lpac,ostream& str)
{
  str << lpac.space() << " Bytes frei " << endl;
  str << "Anzahl Saetze " << lpac.count() << endl;
  for(int i=0;i<lpac.count();i++)
  {
    str << lpac.key(i) << '\t'
	 << lpac.content(i) << endl;
  }
  str << endl;
}

#ifdef LEAFPAGETEST
char speicher[3*PAGESIZE];
int main(int argc,char **argv)
{
  cout << "sizeof(memp_t)=" << sizeof(memp_t) << endl;
  cout << "sizeof(blobptr_t)=" << sizeof(blobptr_t) << endl;
  cout << "sizeof(content_t)=" << sizeof(content_t) << endl;
  cout << "sizeof(Record_t)=" << sizeof(Record_t) << endl;

  RAMS::Mseg sv(speicher,3*PAGESIZE);
  RAMS::RPointer<CommonPage,PAGESIZE> base(0,&sv);
  LeafPageAccess lpac(base.ptr());
  lpac.init();
  cout << lpac.space() << " Bytes frei " << endl;
  cout << "Anzahl Saetze " << lpac.count() << endl;

  char puffer[2048];
  while(fgets(puffer,2048,stdin)!=0)
  {
    char *cont = strchr(puffer,'\t');
    if(cont == NULL) 
    {
      cerr << "Kein Inhalt ..." << endl;
      continue;
    }
    cont++;
    mempl_t key(cont-puffer-1,(octet_t *)puffer);
    mempl_t content(strlen(cont)-1,(octet_t *)cont);
    cerr << "Key :>" << key << '<' << endl;
    cerr << "Content:>" << content << '<' << endl;
    int r=lpac.insert(key.ptr,key.l,content.ptr,content.l);
    cout << r << endl;
    LeafPageAccess_dump(lpac,cout);
  }  

  cout << "Teilen der Seite: " << endl;
  RAMS::RPointer<CommonPage,PAGESIZE> neu1(0,&sv);
  RAMS::RPointer<CommonPage,PAGESIZE> neu2(0,&sv);
  base+=1;
  neu1=base;
  base+=1;
  neu2=base;
  lpac.splitt(neu1.ptr(),neu2.ptr());
  LeafPageAccess erste(neu1.ptr());
  LeafPageAccess zweite(neu2.ptr());
  cout << "---------- Erste Seite --------------" << endl;
  LeafPageAccess_dump(erste,cout);
  cout << "---------- Zweite Seite -------------" << endl;
  LeafPageAccess_dump(zweite,cout); 
}
#endif

