/**
 * @file leafPage.h
 * @brief defines a Leaf page and the Access-Structure
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A> 2002
 *
 *
 */
#ifndef LEAFPAGE_H
#define LEAFPAGE_H
#include"keypage.h"

struct memp_t             // memory Piece in Page, variabel length
{
  ploff_t off;
  uint16_t len;
  octet_t chksum;
};

struct blobptr_t
{
  off_t goff;
};

/**
 * @brief the content of a Record
 * Ein binary lage Object ist hier schon vorgesehen.
 */
struct content_t
{
  union 
  {
    memp_t memp;
    blobptr_t blobptr;
  };
  octet_t is_blob;
};

/**
 * @brief The Record in the Page
 */
struct Record_t
{
  memp_t key;
  content_t content;
};

#include"common_page.h"

/**
 * Layout of a Leaf Page
 */

struct LeafPage : KeyPage
{
  ploff_t last; /// allocationpointer, Memory groes from bottom to top
  Record_t records[1]; // growing
  static  LeafPage *init(char *m);

  ploff_t endoftable()
  { return offsetof(LeafPage,records)+anz*sizeof(Record_t);
  }
};

/**
 * @brief The Access Structure for a leafPage
 */
class LeafPageAccess : public KeyPageAccess
{
public:
  LeafPageAccess():KeyPageAccess() {}
  LeafPageAccess(CommonPage *base)
    :KeyPageAccess(base) {}
  virtual Page_type_t pagetype();
  /**
   * initialisiert leere Seite
   */
  void init()
    {
      LeafPage::init((char *)begin());
    }
  void init(CommonPage *base)
  {
    RAMS::Mseg::setto((char *)base,PAGESIZE);
    init();
  }
  size_t space();

  /**
   * F&uuml;gt Schl&uuml:ssel und Satz ein
   * @return -1 falls voll, index sonst
   */
  int insert(const octet_t *key,size_t keylen,const octet_t *cont,size_t clen); 
  virtual int splitt(CommonPage *neu1,CommonPage *neu2);


protected:
  /**
   * \brief Inserts string
   * @returns >=0 if ok, -1 if not
   */
  int insertstr(memp_t& memp,const octet_t *str,size_t len);
  LeafPage *page() { return (LeafPage *)begin(); }
public:
  int keycmp(int i,const octet_t *other,size_t olen);
  mempl_t key(int i);

  mempl_t content(int i)
  {
    mempl_t ret;
    ret.l = page()->records[i].content.memp.len;
    ret.ptr = (octet_t *)begin()+page()->records[i].content.memp.off;
    return ret;
  }
};
#endif






